import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

class ChatClient extends Frame implements Observer {

	private ChatServer server;
	private String username;
	private TextArea displayArea;
	private TextField inputField;

	public ChatClient(String username, ChatServer server) {
		super("Chat-Client: " + username);
		this.server = server;
		this.username = username;
		this.setLayout(new GridLayout(0, 1));
		this.displayArea = new TextArea(20, 10);
		this.displayArea.setEditable(false);
		this.add(this.displayArea);
		this.inputField = new TextField(80);
		this.inputField.addActionListener(new InputListener(this));
		this.add(this.inputField);

		this.server.addObserver(this);

		this.pack();
		this.setVisible(true);
	}

	public void update(Observable o, Object arg) {
		String newInput = (String) arg;
		this.displayArea.append(newInput + "\n");
	}

	public String getUsername() {
		return this.username;
	}

	public ChatServer getServer() {
		return this.server;
	}

	public TextField getInputField() {
		return this.inputField;
	}

}

class InputListener implements ActionListener {

	private ChatClient client;

	InputListener(ChatClient client) {
		this.client = client;
	}

	public void actionPerformed(ActionEvent e) {
		this.client.getServer().newInput(this.client.getUsername(),
				this.client.getInputField().getText());
		this.client.getInputField().setText("");
	}

}

class ChatServer extends Observable {

	public ChatServer() {
		super();
	}

	public void newInput(String username, String input) {
		this.setChanged();
		this.notifyObservers(username + ": " + input);
	}
}

public class UE20Aufgabe17 {

	public static void main(String[] args) {
		ChatServer server = new ChatServer();
		for (int i = 0; i < args.length; i++) {
			ChatClient client = new ChatClient(args[i], server);
			client.setLocation(i * 100, i * 50);
		}

	}

}
