import java.awt.Button;
import java.awt.Color;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Calendar;

/**
 * Klasse, die Uhrzeiten von Stopp-Uhren reprsentiert. Intern werden dabei
 * Stunden (0-23), Minuten (0  59) und Sekunden (0  59) verwaltet. Beachten
 * werden dabei die bekannten Umrechnungsregeln fr Stunden, Minuten und
 * Sekunden (zum Beispiel: 61 Minuten = 1 Stunde + 1 Minute).
 * 
 * @author Dietrich Boles
 * @version 09.07.2010
 * 
 */
class Clock {

	private int seconds;
	private int minutes;
	private int hours;

	/**
	 * Default-Konstruktor; initialisiert ein Clock-Objekt mit 00:00:00
	 */
	public Clock() {
		this.reset();
	}

	/**
	 * initialisiert das Clock-Objekt mit Werten fr die Stunden, Minuten und
	 * Sekunden, die ihm als Parameter bergeben werden.
	 * 
	 * @param hours
	 *            muss groesser als 0 sein
	 * @param minutes
	 *            muss groesser als 0 sein
	 * @param seconds
	 *            muss groesser als 0 sein
	 */
	public Clock(int hours, int minutes, int seconds) {
		this.seconds = seconds % 60;
		minutes += seconds / 60;
		this.minutes = minutes % 59;
		hours += minutes / 60;
		this.hours = hours % 24;
	}

	/**
	 * Copy-Konstruktor
	 * 
	 * @param clock
	 *            darf nicht null sein
	 */
	public Clock(Clock clock) {
		this.seconds = clock.seconds;
		this.minutes = clock.minutes;
		this.hours = clock.hours;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#clone()
	 */
	@Override
	public Clock clone() {
		return new Clock(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof Clock)) {
			return false;
		}
		Clock clock = (Clock) obj;
		return this.seconds == clock.seconds && this.minutes == clock.minutes
				&& this.hours == clock.hours;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "" + Clock.to2DigitsString(this.hours) + ":"
				+ Clock.to2DigitsString(this.minutes) + ":"
				+ Clock.to2DigitsString(this.seconds);
	}

	/**
	 * Addiert secs Sekunden zur aktuellen Uhrzeit. Beim einem Overflow wird die
	 * Uhrzeit auf 00:00:00 gesetzt.
	 * 
	 * @param secs
	 *            muss groesser als 0 sein
	 */
	public void addSeconds(int secs) {
		this.seconds += secs;
		if (this.seconds >= 60) {
			this.seconds = 0;
			this.minutes++;
			if (this.minutes >= 60) {
				this.minutes = 0;
				this.hours++;
				if (this.hours >= 24) {
					this.reset();
				}
			}
		}
	}

	/**
	 * liefert ein Clock-Objekt mit der Summe der durch die beiden
	 * Parameter-Objekte reprsentierten Uhrzeiten
	 * 
	 * @param clock1
	 *            darf nicht null sein
	 * @param clock2
	 *            darf nicht null sein
	 * @return ein Clock-Objekt mit der Summe der durch die beiden
	 *         Parameter-Objekte reprsentierten Uhrzeiten
	 */
	public static Clock add(Clock clock1, Clock clock2) {
		return new Clock(clock1.hours + clock2.hours, clock1.minutes
				+ clock2.minutes, clock1.seconds + clock2.seconds);
	}

	public void reset() {
		this.seconds = 0;
		this.minutes = 0;
		this.hours = 0;
	}

	private static String to2DigitsString(int value) {
		if (value > 9) {
			return "" + value;
		} else {
			return "0" + value;
		}
	}

}

public class UE20Aufgabe21 {

	public static void main(String[] args) {
		Frame fenster = new Frame("Uhr");
		fenster.setLayout(new GridLayout(0, 1));
		fenster.setSize(600, 600);
		DigitalClock clock = new DigitalClock();
		fenster.add(clock);
		Button farbwechsel = new Button("Farbwechsel");
		farbwechsel.addActionListener(new FarbwechselListener(clock));
		fenster.add(farbwechsel);
		fenster.setVisible(true);
	}
}

class FarbwechselListener implements ActionListener {

	private DigitalClock clock;

	public FarbwechselListener(DigitalClock clock) {
		this.clock = clock;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		this.clock.switchColor();
	}

}

class DigitalClock extends Panel {
	private Color color;
	private ClockLabel clock;
	private DigitalPanel[] dPanel;

	public DigitalClock() {
		this.color = Color.red;
		this.dPanel = new DigitalPanel[4];
		this.clock = new ClockLabel(2, 33, 46);
		this.setLayout(new GridLayout(3, 3));

		this.add(new Panel());
		this.add(this.dPanel[0] = new DigitalPanel(1, this));
		this.add(new Panel());

		this.add(this.dPanel[3] = new DigitalPanel(4, this));
		this.add(this.clock);
		this.add(this.dPanel[1] = new DigitalPanel(2, this));

		this.add(new Panel());
		this.add(this.dPanel[2] = new DigitalPanel(3, this));
		this.add(new Panel());
	}

	public Color getColor() {
		return this.color;
	}

	public ClockLabel getClock() {
		return this.clock;
	}

	public void switchColor() {
		if (this.color == Color.red) {
			this.color = Color.green;
		} else {
			this.color = Color.red;
		}
		for (int i = 0; i < this.dPanel.length; i++) {
			this.dPanel[i].setBackground();
		}
	}
}

class DigitalPanel extends Panel implements ClockListener {
	private int number;
	private DigitalClock dClock;

	public DigitalPanel(int number, DigitalClock c) {
		super();
		this.number = number;
		this.dClock = c;
		this.setBackground();
	}

	public void setBackground() {
		if (this.dClock.getClock().getHours() >= this.number * 3 - 2) {
			this.setBackground(this.dClock.getColor());
		} else {
			this.setBackground(Color.WHITE);
		}
	}

	@Override
	public void clockChanged(ClockEvent evt) {
		this.setBackground();
	}
}



/**
 * Event-Objekt fuer Zustandsaenderungen von ClockLabel-Objekten
 * 
 * @author Dietrich Boles
 * @version 25.05.2011
 */
 class ClockEvent {
	
	private int hours;
	private int minutes;
	private int seconds;

	/**
	 * initialisiert ein ClockEvent mit der aktuellen Stunden-, Minuten- und
	 * Sekundenzahl
	 * 
	 * @param hours
	 *            die aktuelle Stundenzahl
	 * @param min
	 *            die aktuelle Minutenzahl
	 * @param sec
	 *            die aktuelle Sekundenzahl
	 */
	ClockEvent(int hours, int min, int sec) {
		this.hours = hours;
		this.minutes = min;
		this.seconds = sec;
	}

	/**
	 * liefert die aktuelle Stundenzahl
	 * 
	 * @return die aktuelle Stundenzahl
	 */
	public int getHours() {
		return this.hours;
	}

	/**
	 * liefert die aktuelle Minutenzahl
	 * 
	 * @return die aktuelle Minutenzahl
	 */
	public int getMinutes() {
		return this.minutes;
	}

	/**
	 * liefert die aktuelle Sekundenzahl
	 * 
	 * @return die aktuelle Sekundenzahl
	 */
	public int getSeconds() {
		return this.seconds;
	}
}
 


/**
 * Spezialisiertes Java-AWT-Label fuer die Darstellung von Uhrzeiten; Uhrzeiten
 * werden mit Stunden, Minuten und Sekunden repraesentiert und liegen im Bereich
 * von 00:00:00 bis 11:59:59, d.h. Nachmittagszeiten werden auf die
 * Vormittagszeiten abgebildet (zum Beispiel 14:00 Uhr auf 02:00 Uhr).
 * <p>
 * </p>
 * Die Zeitanzeige eines ClockLabel-Objektes, d.h. sein Zustand, wird
 * sekuendlich aktualisiert.
 * 
 * @author Dietrich Boles
 * @version 25.05.2011
 */
 class ClockLabel extends Label {

	private ArrayList<ClockListener> listeners;
	private String time;

	/**
	 * initialisiert ein ClockLabel mit der aktuellen Uhrzeit; die Zeitanzeige
	 * wird sekuendlich aktualisiert
	 */
	public ClockLabel() {
		super();
		super.setAlignment(Label.CENTER);
		this.time = ClockLabel.calcTime();
		this.setText(this.time);
		this.listeners = new ArrayList<ClockListener>();
		Thread th = new ClockThread();
		th.setDaemon(true);
		th.start();
	}

	/**
	 * initialisiert ein ClockLabel mit den uebergebenen Werten; Achtung: Dieser
	 * Konstruktor dient nur zum Testen, die Zeitanzeige wird NICHT
	 * aktualisiert!
	 * 
	 * @param hours
	 *            Stundenzahl (0-11)
	 * @param minutes
	 *            Minutenzahl (0-59)
	 * @param secs
	 *            Sekundenzahl (0-59)
	 */
	public ClockLabel(int hours, int minutes, int secs) {
		super();
		super.setAlignment(Label.CENTER);
		this.time = ClockLabel.getTime(hours, minutes, secs);
		this.setText(this.time);
		this.listeners = new ArrayList<ClockListener>();
	}

	/**
	 * liefert die aktuelle Stundenzahl
	 * 
	 * @return die aktuelle Stundenzahl
	 */
	public int getHours() {
		return new Integer(this.time.substring(0, 2));
	}

	/**
	 * liefert die aktuelle Minutenzahl
	 * 
	 * @return die aktuelle Minutenzahl
	 */
	public int getMinutes() {
		return new Integer(this.time.substring(3, 5));
	}

	/**
	 * liefert die aktuelle Sekundenzahl
	 * 
	 * @return die aktuelle Sekundenzahl
	 */
	public int getSeconds() {
		return new Integer(this.time.substring(6, 8));
	}

	/**
	 * zum Anmelden eines neuen ClockListeners
	 * 
	 * @param l
	 *            der neue ClockListener (!= null)
	 */
	public void addClockListener(ClockListener l) {
		this.listeners.add(l);
	}

	/**
	 * zum Abmelden eines ClockListeners
	 * 
	 * @param l
	 *            ein ClockListener (!= null)
	 */
	public void removeClockListener(ClockListener l) {
		this.listeners.remove(l);
	}

	private static String calcTime() {
		Calendar cal = Calendar.getInstance();
		int hour = cal.get(Calendar.HOUR);
		int min = cal.get(Calendar.MINUTE);
		int sec = cal.get(Calendar.SECOND);
		String result = "";
		if (hour < 10) {
			result += "0";
		}
		result += hour;
		result += ":";
		if (min < 10) {
			result += "0";
		}
		result += min;
		result += ":";
		if (sec < 10) {
			result += "0";
		}
		result += sec;
		return result;
	}

	private static String getTime(int hour, int min, int sec) {
		hour = hour % 12;
		min = min % 60;
		sec = sec % 60;
		String result = "";
		if (hour < 10) {
			result += "0";
		}
		result += hour;
		result += ":";
		if (min < 10) {
			result += "0";
		}
		result += min;
		result += ":";
		if (sec < 10) {
			result += "0";
		}
		result += sec;
		return result;
	}

	class ClockThread extends Thread {
		@Override
		public void run() {
			while (true) {
				String t = ClockLabel.calcTime();
				if (!t.equals(ClockLabel.this.time)) {
					ClockLabel.this.time = t;
					ClockLabel.this.setText(ClockLabel.this.time);
					ClockEvent evt = new ClockEvent(new Integer(
							ClockLabel.this.time.substring(0, 2)), new Integer(
							ClockLabel.this.time.substring(3, 5)), new Integer(
							ClockLabel.this.time.substring(6, 8)));
					for (ClockListener l : ClockLabel.this.listeners) {
						l.clockChanged(evt);
					}
				}
				try {
					Thread.sleep(500);
				} catch (InterruptedException e) {
				}
			}
		}
	}

}

/**
 * Listener fuer Zustandsaenderungen von ClockLabel-Objekten
 * 
 * @author Dietrich Boles
 * @version 25.05.2011
 */
 interface ClockListener {
	/**
	 * wird fuer alle bei einem ClockLabel registrierten ClockListener
	 * aufgerufen, wenn sich der Zustand des ClockLabels aendert
	 * 
	 * @param evt
	 *            enthaelt Informationen zum aufgetretenen Ereignis
	 */
	public abstract void clockChanged(ClockEvent evt);
}


